@@ -44,6 +44,8 @@ import Data.Certificate.X509.Ext
 import qualified Crypto.Types.PubKey.RSA as RSA
 import qualified Crypto.Types.PubKey.DSA as DSA
 
+import Data.Certificate.KeyRSA (parse_RSA)
+
 data HashALG =
 	  HashMD2
 	| HashMD5
@@ -110,22 +112,6 @@ oidCountry          = [2,5,4,6]
 oidOrganization     = [2,5,4,10]
 oidOrganizationUnit = [2,5,4,11]
 
-{- | parse a RSA pubkeys from ASN1 encoded bits.
- - return PubKeyRSA (len-modulus, modulus, e) if successful -}
-parse_RSA :: ByteString -> ParseASN1 PubKey
-parse_RSA bits =
-	case decodeASN1Stream $ bits of
-		Right [Start Sequence, IntVal modulus, IntVal pubexp, End Sequence] ->
-			return $ PubKeyRSA $ RSA.PublicKey
-				{ RSA.public_size = calculate_modulus modulus 1
-				, RSA.public_n    = modulus
-				, RSA.public_e    = pubexp
-				}
-		_ ->
-			throwError ("bad RSA format")
-	where
-		calculate_modulus n i = if (2 ^ (i * 8)) > n then i else calculate_modulus n (i+1)
-
 parse_ECDSA :: ByteString -> ParseASN1 PubKey
 parse_ECDSA bits =
 	case decodeASN1Stream bits of
@@ -249,7 +235,7 @@ parseCertHeaderSubjectPK = onNextContainer Sequence $ do
 		[OID pkalg,Null] -> do
 			let sig = oidPubKey pkalg
 			case sig of
-				PubKeyALG_RSA -> parse_RSA bits
+				PubKeyALG_RSA -> either (throwError) (return . PubKeyRSA) (parse_RSA bits)
 				_             -> return $ PubKeyUnknown pkalg $ L.unpack bits
 		[OID pkalg,OID _] -> do
 			let sig = oidPubKey pkalg
